@@ -6,8 +6,8 @@ use cap_primitives::{
     ambient_authority,
     fs::{open_dir_nofollow, set_times, set_times_nofollow},
 };
+use io_lifetimes::AsFilelike;
 use std::{io, path::Path};
-use unsafe_io::AsUnsafeFile;
 
 pub use cap_primitives::fs::SystemTimeSpec;
 
@@ -180,12 +180,22 @@ pub trait DirExtUtf8 {
 impl DirExt for cap_std::fs::Dir {
     #[inline]
     fn set_atime<P: AsRef<Path>>(&self, path: P, atime: SystemTimeSpec) -> io::Result<()> {
-        set_times(&self.as_file_view(), path.as_ref(), Some(atime), None)
+        set_times(
+            &self.as_filelike_view::<std::fs::File>(),
+            path.as_ref(),
+            Some(atime),
+            None,
+        )
     }
 
     #[inline]
     fn set_mtime<P: AsRef<Path>>(&self, path: P, mtime: SystemTimeSpec) -> io::Result<()> {
-        set_times(&self.as_file_view(), path.as_ref(), None, Some(mtime))
+        set_times(
+            &self.as_filelike_view::<std::fs::File>(),
+            path.as_ref(),
+            None,
+            Some(mtime),
+        )
     }
 
     #[inline]
@@ -195,7 +205,12 @@ impl DirExt for cap_std::fs::Dir {
         atime: Option<SystemTimeSpec>,
         mtime: Option<SystemTimeSpec>,
     ) -> io::Result<()> {
-        set_times(&self.as_file_view(), path.as_ref(), atime, mtime)
+        set_times(
+            &self.as_filelike_view::<std::fs::File>(),
+            path.as_ref(),
+            atime,
+            mtime,
+        )
     }
 
     #[inline]
@@ -205,13 +220,22 @@ impl DirExt for cap_std::fs::Dir {
         atime: Option<SystemTimeSpec>,
         mtime: Option<SystemTimeSpec>,
     ) -> io::Result<()> {
-        set_times_nofollow(&self.as_file_view(), path.as_ref(), atime, mtime)
+        set_times_nofollow(
+            &self.as_filelike_view::<std::fs::File>(),
+            path.as_ref(),
+            atime,
+            mtime,
+        )
     }
 
     #[cfg(not(windows))]
     #[inline]
     fn symlink<P: AsRef<Path>, Q: AsRef<Path>>(&self, src: P, dst: Q) -> io::Result<()> {
-        symlink(src.as_ref(), &self.as_file_view(), dst.as_ref())
+        symlink(
+            src.as_ref(),
+            &self.as_filelike_view::<std::fs::File>(),
+            dst.as_ref(),
+        )
     }
 
     #[cfg(not(windows))]
@@ -239,18 +263,26 @@ impl DirExt for cap_std::fs::Dir {
     #[cfg(windows)]
     #[inline]
     fn symlink_file<P: AsRef<Path>, Q: AsRef<Path>>(&self, src: P, dst: Q) -> io::Result<()> {
-        symlink_file(src.as_ref(), &self.as_file_view(), dst.as_ref())
+        symlink_file(
+            src.as_ref(),
+            &self.as_filelike_view::<std::fs::File>(),
+            dst.as_ref(),
+        )
     }
 
     #[cfg(windows)]
     #[inline]
     fn symlink_dir<P: AsRef<Path>, Q: AsRef<Path>>(&self, src: P, dst: Q) -> io::Result<()> {
-        symlink_dir(src.as_ref(), &self.as_file_view(), dst.as_ref())
+        symlink_dir(
+            src.as_ref(),
+            &self.as_filelike_view::<std::fs::File>(),
+            dst.as_ref(),
+        )
     }
 
     #[inline]
     fn open_dir_nofollow<P: AsRef<Path>>(&self, path: P) -> io::Result<Self> {
-        match open_dir_nofollow(&self.as_file_view(), path.as_ref()) {
+        match open_dir_nofollow(&self.as_filelike_view::<std::fs::File>(), path.as_ref()) {
             Ok(file) => Ok(Self::from_std_file(file, ambient_authority())),
             Err(e) => Err(e),
         }
@@ -304,12 +336,22 @@ impl DirExt for cap_std::fs::Dir {
 impl DirExt for cap_async_std::fs::Dir {
     #[inline]
     fn set_atime<P: AsRef<Path>>(&self, path: P, atime: SystemTimeSpec) -> io::Result<()> {
-        set_times(&self.as_file_view(), path.as_ref(), Some(atime), None)
+        set_times(
+            &self.as_filelike_view::<std::fs::File>(),
+            path.as_ref(),
+            Some(atime),
+            None,
+        )
     }
 
     #[inline]
     fn set_mtime<P: AsRef<Path>>(&self, path: P, mtime: SystemTimeSpec) -> io::Result<()> {
-        set_times(&self.as_file_view(), path.as_ref(), None, Some(mtime))
+        set_times(
+            &self.as_filelike_view::<std::fs::File>(),
+            path.as_ref(),
+            None,
+            Some(mtime),
+        )
     }
 
     #[inline]
@@ -319,7 +361,12 @@ impl DirExt for cap_async_std::fs::Dir {
         atime: Option<SystemTimeSpec>,
         mtime: Option<SystemTimeSpec>,
     ) -> io::Result<()> {
-        set_times(&self.as_file_view(), path.as_ref(), atime, mtime)
+        set_times(
+            &self.as_filelike_view::<std::fs::File>(),
+            path.as_ref(),
+            atime,
+            mtime,
+        )
     }
 
     #[inline]
@@ -329,13 +376,22 @@ impl DirExt for cap_async_std::fs::Dir {
         atime: Option<SystemTimeSpec>,
         mtime: Option<SystemTimeSpec>,
     ) -> io::Result<()> {
-        set_times_nofollow(&self.as_file_view(), path.as_ref(), atime, mtime)
+        set_times_nofollow(
+            &self.as_filelike_view::<std::fs::File>(),
+            path.as_ref(),
+            atime,
+            mtime,
+        )
     }
 
     #[cfg(not(windows))]
     #[inline]
     fn symlink<P: AsRef<Path>, Q: AsRef<Path>>(&self, src: P, dst: Q) -> io::Result<()> {
-        symlink(src.as_ref(), &self.as_file_view(), dst.as_ref())
+        symlink(
+            src.as_ref(),
+            &self.as_filelike_view::<std::fs::File>(),
+            dst.as_ref(),
+        )
     }
 
     #[cfg(not(windows))]
@@ -363,18 +419,26 @@ impl DirExt for cap_async_std::fs::Dir {
     #[cfg(windows)]
     #[inline]
     fn symlink_file<P: AsRef<Path>, Q: AsRef<Path>>(&self, src: P, dst: Q) -> io::Result<()> {
-        symlink_file(src.as_ref(), &self.as_file_view(), dst.as_ref())
+        symlink_file(
+            src.as_ref(),
+            &self.as_filelike_view::<std::fs::File>(),
+            dst.as_ref(),
+        )
     }
 
     #[cfg(windows)]
     #[inline]
     fn symlink_dir<P: AsRef<Path>, Q: AsRef<Path>>(&self, src: P, dst: Q) -> io::Result<()> {
-        symlink_dir(src.as_ref(), &self.as_file_view(), dst.as_ref())
+        symlink_dir(
+            src.as_ref(),
+            &self.as_filelike_view::<std::fs::File>(),
+            dst.as_ref(),
+        )
     }
 
     #[inline]
     fn open_dir_nofollow<P: AsRef<Path>>(&self, path: P) -> io::Result<Self> {
-        match open_dir_nofollow(&self.as_file_view(), path.as_ref()) {
+        match open_dir_nofollow(&self.as_filelike_view::<std::fs::File>(), path.as_ref()) {
             Ok(file) => Ok(Self::from_std_file(file.into(), ambient_authority())),
             Err(e) => Err(e),
         }
@@ -428,13 +492,23 @@ impl DirExtUtf8 for cap_std::fs_utf8::Dir {
     #[inline]
     fn set_atime<P: AsRef<str>>(&self, path: P, atime: SystemTimeSpec) -> io::Result<()> {
         let path = from_utf8(path)?;
-        set_times(&self.as_file_view(), &path, Some(atime), None)
+        set_times(
+            &self.as_filelike_view::<std::fs::File>(),
+            &path,
+            Some(atime),
+            None,
+        )
     }
 
     #[inline]
     fn set_mtime<P: AsRef<str>>(&self, path: P, mtime: SystemTimeSpec) -> io::Result<()> {
         let path = from_utf8(path)?;
-        set_times(&self.as_file_view(), &path, None, Some(mtime))
+        set_times(
+            &self.as_filelike_view::<std::fs::File>(),
+            &path,
+            None,
+            Some(mtime),
+        )
     }
 
     #[inline]
@@ -445,7 +519,12 @@ impl DirExtUtf8 for cap_std::fs_utf8::Dir {
         mtime: Option<SystemTimeSpec>,
     ) -> io::Result<()> {
         let path = from_utf8(path)?;
-        set_times(&self.as_file_view(), &path, atime, mtime)
+        set_times(
+            &self.as_filelike_view::<std::fs::File>(),
+            &path,
+            atime,
+            mtime,
+        )
     }
 
     #[inline]
@@ -456,7 +535,12 @@ impl DirExtUtf8 for cap_std::fs_utf8::Dir {
         mtime: Option<SystemTimeSpec>,
     ) -> io::Result<()> {
         let path = from_utf8(path)?;
-        set_times_nofollow(&self.as_file_view(), &path, atime, mtime)
+        set_times_nofollow(
+            &self.as_filelike_view::<std::fs::File>(),
+            &path,
+            atime,
+            mtime,
+        )
     }
 
     #[cfg(not(windows))]
@@ -501,7 +585,10 @@ impl DirExtUtf8 for cap_std::fs_utf8::Dir {
 
     #[inline]
     fn open_dir_nofollow<P: AsRef<str>>(&self, path: P) -> io::Result<Self> {
-        match open_dir_nofollow(&self.as_file_view(), path.as_ref().as_ref()) {
+        match open_dir_nofollow(
+            &self.as_filelike_view::<std::fs::File>(),
+            path.as_ref().as_ref(),
+        ) {
             Ok(file) => Ok(Self::from_std_file(file.into(), ambient_authority())),
             Err(e) => Err(e),
         }
@@ -555,13 +642,23 @@ impl DirExtUtf8 for cap_async_std::fs_utf8::Dir {
     #[inline]
     fn set_atime<P: AsRef<str>>(&self, path: P, atime: SystemTimeSpec) -> io::Result<()> {
         let path = from_utf8(path)?;
-        set_times(&self.as_file_view(), &path, Some(atime), None)
+        set_times(
+            &self.as_filelike_view::<std::fs::File>(),
+            &path,
+            Some(atime),
+            None,
+        )
     }
 
     #[inline]
     fn set_mtime<P: AsRef<str>>(&self, path: P, mtime: SystemTimeSpec) -> io::Result<()> {
         let path = from_utf8(path)?;
-        set_times(&self.as_file_view(), &path, None, Some(mtime))
+        set_times(
+            &self.as_filelike_view::<std::fs::File>(),
+            &path,
+            None,
+            Some(mtime),
+        )
     }
 
     #[inline]
@@ -572,7 +669,12 @@ impl DirExtUtf8 for cap_async_std::fs_utf8::Dir {
         mtime: Option<SystemTimeSpec>,
     ) -> io::Result<()> {
         let path = from_utf8(path)?;
-        set_times(&self.as_file_view(), &path, atime, mtime)
+        set_times(
+            &self.as_filelike_view::<std::fs::File>(),
+            &path,
+            atime,
+            mtime,
+        )
     }
 
     #[inline]
@@ -583,7 +685,12 @@ impl DirExtUtf8 for cap_async_std::fs_utf8::Dir {
         mtime: Option<SystemTimeSpec>,
     ) -> io::Result<()> {
         let path = from_utf8(path)?;
-        set_times_nofollow(&self.as_file_view(), &path, atime, mtime)
+        set_times_nofollow(
+            &self.as_filelike_view::<std::fs::File>(),
+            &path,
+            atime,
+            mtime,
+        )
     }
 
     #[cfg(not(windows))]
@@ -591,7 +698,7 @@ impl DirExtUtf8 for cap_async_std::fs_utf8::Dir {
     fn symlink<P: AsRef<str>, Q: AsRef<str>>(&self, src: P, dst: Q) -> io::Result<()> {
         let src = from_utf8(src)?;
         let dst = from_utf8(dst)?;
-        symlink(&src, &self.as_file_view(), &dst)
+        symlink(&src, &self.as_filelike_view::<std::fs::File>(), &dst)
     }
 
     #[cfg(not(windows))]
@@ -621,7 +728,7 @@ impl DirExtUtf8 for cap_async_std::fs_utf8::Dir {
     fn symlink_file<P: AsRef<str>, Q: AsRef<str>>(&self, src: P, dst: Q) -> io::Result<()> {
         let src = from_utf8(src)?;
         let dst = from_utf8(dst)?;
-        symlink_file(&src, &self.as_file_view(), &dst)
+        symlink_file(&src, &self.as_filelike_view::<std::fs::File>(), &dst)
     }
 
     #[cfg(windows)]
@@ -629,12 +736,15 @@ impl DirExtUtf8 for cap_async_std::fs_utf8::Dir {
     fn symlink_dir<P: AsRef<str>, Q: AsRef<str>>(&self, src: P, dst: Q) -> io::Result<()> {
         let src = from_utf8(src)?;
         let dst = from_utf8(dst)?;
-        symlink_dir(&src, &self.as_file_view(), &dst)
+        symlink_dir(&src, &self.as_filelike_view::<std::fs::File>(), &dst)
     }
 
     #[inline]
     fn open_dir_nofollow<P: AsRef<str>>(&self, path: P) -> io::Result<Self> {
-        match open_dir_nofollow(&self.as_file_view(), path.as_ref().as_ref()) {
+        match open_dir_nofollow(
+            &self.as_filelike_view::<std::fs::File>(),
+            path.as_ref().as_ref(),
+        ) {
             Ok(file) => Ok(Self::from_std_file(file.into(), ambient_authority())),
             Err(e) => Err(e),
         }
